'use strict';

var WebSocket = require('isomorphic-ws');

function WebSocketClient() {
	var autoReconnect = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

	this.number = 0; // Message number
	this.autoReconnect = autoReconnect;
	this.autoReconnectInterval = 5 * 1000; // ms
}

WebSocketClient.prototype.open = function (url) {
	var _this = this;

	this.url = url;
	this.instance = new WebSocket(this.url);
	this.instance.onopen = function () {
		_this.onopen();
	};
	this.instance.onmessage = function (data, flags) {
		_this.number++;
		_this.onmessage(data, flags, _this.number);
	};
	this.instance.onclose = function (e) {
		switch (e.code) {
			case 1000:
				// CLOSE_NORMAL
				console.log("WebSocket: closed");
				break;
			default:
				// Abnormal closure
				_this.reconnect(e);
				break;
		}
		_this.onclose(e);
	};
	this.instance.onerror = function (e) {
		switch (e.code) {
			case 'ECONNREFUSED':
				_this.reconnect(e);
				break;
			default:
				_this.onerror(e);
				break;
		}
	};
};

WebSocketClient.prototype.send = function (data, option) {
	try {
		this.instance.send(data, option);
	} catch (e) {
		this.instance.emit('error', e);
	}
};

WebSocketClient.prototype.reconnect = function (e) {
	if (!this.autoReconnect) {
		console.log("WebSocket: closed");
		return;
	}
	console.log('WebSocketClient: retry in ' + this.autoReconnectInterval + 'ms', e);
	this.instance.removeAllListeners();
	var that = this;
	setTimeout(function () {
		console.log("WebSocketClient: reconnecting...");
		that.open(that.url);
	}, this.autoReconnectInterval);
};

WebSocketClient.prototype.onopen = function (e) {
	console.log("WebSocketClient: open", arguments);
};
WebSocketClient.prototype.onmessage = function (data, flags, number) {
	console.log("WebSocketClient: message", arguments);
};
WebSocketClient.prototype.onerror = function (e) {
	console.log("WebSocketClient: error", arguments);
};
WebSocketClient.prototype.onclose = function (e) {
	console.log("WebSocketClient: closed", arguments);
};

module.exports = WebSocketClient;